
/* Chrysalide - Outil d'analyse de fichiers binaires
 * module.c - intégration du répertoire types en tant que module
 *
 * Copyright (C) 2014-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "module.h"


#include <assert.h>


#include "array.h"
#include "basic.h"
#include "cse.h"
#include "encaps.h"
#include "expr.h"
#include "literal.h"
#include "override.h"
#include "proto.h"
#include "template.h"
#include "../../access.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Ajoute le module 'types' au module Python.                   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool add_analysis_types_module_to_python_module(PyObject *super)
{
    bool result;                            /* Bilan à retourner           */
    PyObject *module;                       /* Sous-module mis en place    */
    int ret;                                /* Bilan d'un appel            */

    static PyModuleDef py_chrysalide_types_module = {

        .m_base = PyModuleDef_HEAD_INIT,

        .m_name = "pychrysalide.analysis.types",
        .m_doc = "Python module for Chrysalide.analysis.types",

        .m_size = -1,

    };

    result = false;

    module = PyModule_Create(&py_chrysalide_types_module);
    if (module == NULL) return false;

    ret = PyState_AddModule(super, &py_chrysalide_types_module);
    if (ret != 0) goto loading_failed;

    ret = _PyImport_FixupBuiltin(module, "pychrysalide.analysis.types");
    if (ret != 0) goto loading_failed;

    Py_INCREF(module);
    ret = PyModule_AddObject(super, "types", module);
    if (ret != 0) goto loading_failed;

    result = register_python_array_type(module);
    if (result) result = register_python_basic_type(module);
    if (result) result = register_python_class_enum_type(module);
    if (result) result = register_python_expr_type(module);
    if (result) result = register_python_encapsulated_type(module);
    if (result) result = register_python_literal_type(module);
    if (result) result = register_python_override_type(module);
    if (result) result = register_python_proto_type(module);
    if (result) result = register_python_template_type(module);

    if (result)
        register_access_to_python_module("pychrysalide.analysis.types", module);

 loading_failed:

    assert(result);

    return result;

}
